@@ -1,22 +1,18 @@
 import traceback
-from contextlib import contextmanager
 from enum import Enum
-from typing import Any, Callable, Dict, Generator, Iterable, List, Optional, Tuple, Union, cast
-from urllib.parse import urlparse
+from typing import Callable, Dict, Generator, Iterable, List, Optional, Tuple, Union, cast
 
 import click
 import hypothesis
 import requests
-from requests import exceptions
 
 from .. import checks as checks_module
-from .. import models, runner, utils
-from ..exceptions import HTTPError
-from ..loaders import from_path, from_uri, get_loader_for_app
+from .. import models, runner
 from ..runner import events
 from ..types import Filter
-from ..utils import WSGIResponse, dict_not_none_values, dict_true_values
+from ..utils import WSGIResponse
 from . import callbacks, output
+from .context import ExecutionContext
 from .options import CSVOption, NotSet, OptionalInt
 
 CONTEXT_SETTINGS = {"help_option_names": ["-h", "--help"]}
@@ -160,7 +156,7 @@ def run(  # pylint: disable=too-many-arguments
     tags: Optional[Filter] = None,
     workers_num: int = DEFAULT_WORKERS,
     base_url: Optional[str] = None,
-    app: Any = None,
+    app: Optional[str] = None,
     request_timeout: Optional[int] = None,
     validate_schema: bool = True,
     show_errors_tracebacks: bool = False,
@@ -184,40 +180,39 @@ def run(  # pylint: disable=too-many-arguments
     else:
         selected_checks = tuple(check for check in checks_module.ALL_CHECKS if check.__name__ in checks)
 
-    with abort_on_network_errors():
-        prepared_runner = runner.prepare(
-            schema,
-            auth=auth,
-            auth_type=auth_type,
-            headers=headers,
-            request_timeout=request_timeout,
-            base_url=base_url,
-            endpoint=endpoints,
-            method=methods,
-            tag=tags,
-            app=app,
-            seed=hypothesis_seed,
-            exit_first=exit_first,
-            checks=selected_checks,
-            workers_num=workers_num,
-            validate_schema=validate_schema,
-            hypothesis_deadline=hypothesis_deadline,
-            hypothesis_derandomize=hypothesis_derandomize,
-            hypothesis_max_examples=hypothesis_max_examples,
-            hypothesis_phases=hypothesis_phases,
-            hypothesis_report_multiple_bugs=hypothesis_report_multiple_bugs,
-            hypothesis_suppress_health_check=hypothesis_suppress_health_check,
-            hypothesis_verbosity=hypothesis_verbosity,
-        )
+    prepared_runner = runner.prepare(
+        schema,
+        auth=auth,
+        auth_type=auth_type,
+        headers=headers,
+        request_timeout=request_timeout,
+        base_url=base_url,
+        endpoint=endpoints,
+        method=methods,
+        tag=tags,
+        app=app,
+        seed=hypothesis_seed,
+        exit_first=exit_first,
+        checks=selected_checks,
+        workers_num=workers_num,
+        validate_schema=validate_schema,
+        hypothesis_deadline=hypothesis_deadline,
+        hypothesis_derandomize=hypothesis_derandomize,
+        hypothesis_max_examples=hypothesis_max_examples,
+        hypothesis_phases=hypothesis_phases,
+        hypothesis_report_multiple_bugs=hypothesis_report_multiple_bugs,
+        hypothesis_suppress_health_check=hypothesis_suppress_health_check,
+        hypothesis_verbosity=hypothesis_verbosity,
+    )
     execute(prepared_runner, workers_num, show_errors_tracebacks)
 
 
-def get_output_handler(workers_num: int) -> Callable[[events.ExecutionContext, events.ExecutionEvent], None]:
+def get_output_handler(workers_num: int) -> Callable[[ExecutionContext, events.ExecutionEvent], None]:
     if workers_num > 1:
         output_style = OutputStyle.short
     else:
         output_style = OutputStyle.default
-    return cast(Callable[[events.ExecutionContext, events.ExecutionEvent], None], output_style)
+    return cast(Callable[[ExecutionContext, events.ExecutionEvent], None], output_style)
 
 
 def load_hook(module_name: str) -> None:
@@ -231,24 +226,6 @@ def load_hook(module_name: str) -> None:
         raise click.Abort()
 
 
-@contextmanager
-def abort_on_network_errors() -> Generator[None, None, None]:
-    """Abort on network errors during the schema loading."""
-    try:
-        yield
-    except exceptions.ConnectionError as exc:
-        click.secho(f"Failed to load schema from {exc.request.url}", fg="red")
-        message = utils.format_exception(exc)
-        click.secho(f"Error: {message}", fg="red")
-        raise click.Abort
-    except HTTPError as exc:
-        if exc.response.status_code == 404:
-            click.secho(f"Schema was not found at {exc.url}", fg="red")
-            raise click.Abort
-        click.secho(f"Failed to load schema, code {exc.response.status_code} was returned from {exc.url}", fg="red")
-        raise click.Abort
-
-
 class OutputStyle(Enum):
     """Provide different output styles."""
 
@@ -261,6 +238,6 @@ def execute(
 ) -> None:
     """Execute a prepared runner by drawing events from it and passing to a proper handler."""
     handler = get_output_handler(workers_num)
-    context = events.ExecutionContext(workers_num=workers_num, show_errors_tracebacks=show_errors_tracebacks)
+    context = ExecutionContext(workers_num=workers_num, show_errors_tracebacks=show_errors_tracebacks)
     for event in prepared_runner:
         handler(context, event)
